package com.landray.kmss.sys.attachment.service.spring;

import com.landray.kmss.sys.organization.model.SysOrgPerson;
import com.landray.kmss.sys.right.interfaces.ExtendAuthTmpModel;
import edu.emory.mathcs.backport.java.util.Arrays;
import com.landray.kmss.sys.organization.model.SysOrgElement;
import org.apache.commons.lang3.BooleanUtils;
import com.landray.kmss.sys.right.interfaces.ExtendAuthModel;
import java.util.List;
import java.io.File;
import com.landray.kmss.framework.util.PluginConfigLocationsUtil;
import com.landray.kmss.sys.attachment.borrow.service.ISysAttBorrowBusinessModuleValidator;
import java.lang.reflect.Method;
import org.springframework.util.ReflectionUtils;
import com.landray.kmss.util.ClassUtils;
import org.apache.commons.beanutils.PropertyUtils;
import com.landray.kmss.framework.plugin.core.config.IExtension;
import com.landray.kmss.common.model.IBaseModel;
import com.landray.kmss.util.SpringBeanUtil;
import com.landray.kmss.sys.authentication.intercept.IAuthenticationValidateCore;
import com.landray.kmss.sys.attachment.borrow.service.spring.SysAttBorrowValidatorFactory;
import com.landray.kmss.util.ModelUtil;
import com.landray.kmss.sys.profile.util.ProfileMenuUtil;
import com.landray.kmss.util.UserUtil;
import com.landray.kmss.framework.service.plugin.Plugin;
import com.landray.kmss.sys.attachment.model.SysAttMain;
import com.landray.kmss.sys.attachment.model.SysAttRtfData;
import com.landray.kmss.util.StringUtil;
import com.landray.kmss.sys.authentication.intercept.ValidatorRequestContext;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;
import com.landray.kmss.sys.attachment.service.ISysAttMainCoreInnerService;
import com.landray.kmss.sys.authentication.intercept.IAuthenticationValidator;

public class SysAttachmentValidator implements IAuthenticationValidator
{
    private ISysAttMainCoreInnerService sysAttMainService;
    private Logger logger;
    
    public SysAttachmentValidator() {
        this.logger = LoggerFactory.getLogger(SysAttachmentValidator.class);
    }
    
    public boolean validate(final ValidatorRequestContext validatorContext) throws Exception {
        String method = validatorContext.getParameter("method");
        if ("editDownload".equals(method) || "handleAttUpload".equals(method) || "handleAttToken".equals(method) || "addFile".equals(method) || "getWpsCloudViewUrl".equals(method) || "findAttMains".equals(method) || "getWpsoaassistConfig".equals(method) || "getAttMainViewUrl".equals(method) || "checkUseReadOLConfig".equals(method)) {
            return true;
        }
        String fdId = validatorContext.getParameter("fdId");
        final String fdType = validatorContext.getParameter("fdType");
        final String filethumb = validatorContext.getParameter("filethumb");
        if ("save".equals(method)) {
            final String fdKey = validatorContext.getParameter("fdKey");
            if (StringUtil.isNotNull(fdKey) && fdKey.startsWith("FlowAtt_")) {
                method = "view";
            }
        }
        if ("downloadToOther".equals(method)) {
            method = "download";
        }
        final String fdModelId = validatorContext.getParameter("fdModelId");
        final String fdModelName = validatorContext.getParameter("fdModelName");
        String url = null;
        IBaseModel mainModel = null;
        String authType = null;
        String attType = null;
        String contentType = null;
        String fileName = null;
        SysAttMain attMain = null;
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("开始校验附件权限,requesturl=" + validatorContext.getRequestUrl());
        }
        if (fdType != null && "rtf".equals(fdType)) {
            final SysAttRtfData sysAttRtfData = (SysAttRtfData)this.sysAttMainService.findByPrimaryKey(fdId, (Object)SysAttRtfData.class, true);
            if (sysAttRtfData != null) {
                mainModel = this.sysAttMainService.findByPrimaryKey(sysAttRtfData.getFdModelId(), (Object)sysAttRtfData.getFdModelName(), true);
                if ("view".equals(method) && "yes".equals(filethumb)) {
                    return true;
                }
            }
        }
        if (StringUtil.isNotNull(fdId)) {
            if (fdId.indexOf(";") > -1) {
                fdId = fdId.substring(0, fdId.indexOf(";"));
            }
            if (this.logger.isDebugEnabled()) {
                this.logger.debug("开始校验附件权限,fdId=" + fdId);
            }
            attMain = (SysAttMain)this.sysAttMainService.findByPrimaryKey(fdId, (Object)SysAttMain.class, true);
            if (attMain != null) {
                final IExtension extension = Plugin.getExtension("com.landray.kmss.sys.attachment.validator", attMain.getFdModelName(), "validator");
                if (extension != null) {
                    final IAuthenticationValidator validator = (IAuthenticationValidator)Plugin.getParamValue(extension, "bean");
                    return validator.validate(validatorContext);
                }
                if (StringUtil.isNotNull(attMain.getFdModelId()) && "mobile_online_create".equals(attMain.getFdModelId()) && StringUtil.isNotNull(attMain.getFdCreatorId()) && UserUtil.getUser().getFdId().equals(attMain.getFdCreatorId())) {
                    return true;
                }
                if (StringUtil.isNotNull(attMain.getFdTempKey()) && attMain.getFdTempKey().endsWith("online_create") && StringUtil.isNotNull(attMain.getFdCreatorId()) && UserUtil.getUser().getFdId().equals(attMain.getFdCreatorId())) {
                    return true;
                }
                if (StringUtil.isNotNull(attMain.getFdTempKey()) && attMain.getFdTempKey().endsWith("online_create") && StringUtil.isNotNull(attMain.getFdCreatorId()) && UserUtil.getUser().getFdId().equals(attMain.getFdCreatorId())) {
                    return true;
                }
                if (("com.landray.kmss.km.agreement.model.KmAgreementModel".equals(attMain.getFdModelName()) || "com.landray.kmss.km.sample.model.KmSampleModel".equals(attMain.getFdModelName())) && "mainOnline".equals(attMain.getFdKey()) && ProfileMenuUtil.moduleExist("/km/clause") && StringUtil.isNotNull(attMain.getFdCreatorId()) && UserUtil.getUser().getFdId().equals(attMain.getFdCreatorId())) {
                    return true;
                }
                attType = attMain.getFdAttType();
                if (StringUtil.isNotNull(attMain.getFdModelId()) && StringUtil.isNotNull(attMain.getFdModelName())) {
                    mainModel = this.sysAttMainService.findByPrimaryKey(attMain.getFdModelId(), (Object)attMain.getFdModelName(), true);
                }
                contentType = attMain.getFdContentType();
                fileName = attMain.getFdFileName();
            }
        }
        if (mainModel == null && StringUtil.isNotNull(fdModelId) && StringUtil.isNotNull(fdModelName)) {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug("开始校验附件权限,fdModelId=" + fdModelId + ",fdModelName=" + fdModelName);
            }
            mainModel = this.sysAttMainService.findByPrimaryKey(fdModelId, (Object)fdModelName, true);
        }
        if (mainModel != null) {
            url = ModelUtil.getModelUrl((Object)mainModel);
        }
        if ("save".equals(method) || "update".equals(method)) {
            if (mainModel == null) {
                return true;
            }
        }
        else if ("edit".equals(method)) {
            if (mainModel == null) {
                return true;
            }
            authType = "edit";
        }
        else if ("download".equals(method) || "downloadPdf".equals(method) || "downloadOfd".equals(method) || "readDownload".equals(method)) {
            if (mainModel == null) {
                return (StringUtil.isNotNull(attType) && "pic".equals(attType)) || (attMain != null && StringUtil.isNotNull(attMain.getFdCreatorId()) && UserUtil.getUser().getFdId().equals(attMain.getFdCreatorId()));
            }
            if (!"readDownload".equals(method) || !this.isReadDownload(attType, contentType, fileName)) {
                boolean auth = this.checkModelAuth(mainModel, "authAttNodownload", "authAttDownloads", attType);
                if (!auth) {
                    String downLoadUrl = StringUtil.replace(url, "method=view", "method=edit");
                    downLoadUrl = this.processModelingUrl(mainModel, downLoadUrl);
                    auth = UserUtil.checkAuthentication(downLoadUrl, "GET");
                }
                if (!auth) {
                    auth = SysAttBorrowValidatorFactory.auth(method, fdId);
                }
                if (!auth) {
                    attMain = (SysAttMain)this.sysAttMainService.findByPrimaryKey(fdId);
                    auth = this.checkBusinessModuleBorrowAuth(validatorContext, attMain);
                }
                return auth;
            }
            method = "view";
            authType = "view";
        }
        else if ("print".equals(method)) {
            if (mainModel == null) {
                return false;
            }
            boolean auth = this.checkModelAuth(mainModel, "authAttNoprint", "authAttPrints", "");
            if (!auth) {
                String downLoadUrl = StringUtil.replace(url, "method=view", "method=edit");
                downLoadUrl = this.processModelingUrl(mainModel, downLoadUrl);
                auth = UserUtil.checkAuthentication(downLoadUrl, "GET");
            }
            if (!auth) {
                auth = SysAttBorrowValidatorFactory.auth(method, fdId);
            }
            if (!auth) {
                attMain = (SysAttMain)this.sysAttMainService.findByPrimaryKey(fdId);
                auth = this.checkBusinessModuleBorrowAuth(validatorContext, attMain);
            }
            return auth;
        }
        else if ("copy".equals(method)) {
            if (mainModel == null) {
                return false;
            }
            boolean auth = this.checkModelAuth(mainModel, "authAttNocopy", "authAttCopys", "");
            if (!auth) {
                String downLoadUrl = StringUtil.replace(url, "method=view", "method=edit");
                downLoadUrl = this.processModelingUrl(mainModel, downLoadUrl);
                auth = UserUtil.checkAuthentication(downLoadUrl, "GET");
            }
            if (!auth) {
                auth = SysAttBorrowValidatorFactory.auth(method, fdId);
            }
            if (!auth) {
                attMain = (SysAttMain)this.sysAttMainService.findByPrimaryKey(fdId);
                auth = this.checkBusinessModuleBorrowAuth(validatorContext, attMain);
            }
            return auth;
        }
        else if ("viewDownload".equals(method)) {
            method = "view";
            authType = "view";
        }
        else if ("delete".equals(method) || "deleteall".equals(method)) {
            if (mainModel == null) {
                return true;
            }
            authType = "edit";
        }
        if (mainModel == null) {
            if (StringUtil.isNull(fdId)) {
                return true;
            }
            attMain = (SysAttMain)this.sysAttMainService.findByPrimaryKey(fdId);
            return UserUtil.getUser().getFdId().equals(attMain.getFdCreatorId());
        }
        else {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug("开始校验附件的主model的URL权限,url=" + url);
            }
            if (authType != null) {
                url = StringUtil.replace(url, "method=view", "method=" + authType);
            }
            if (this.logger.isDebugEnabled()) {
                this.logger.debug("开始校验附件的主model的URL权限,url=" + url);
            }
            if (url == null) {
                this.logger.warn(String.valueOf(ModelUtil.getModelClassName((Object)mainModel)) + "对应的数据字典没有配置URL，附件权限校验失败！");
                return false;
            }
            if (!"edit".equals(authType)) {
                Boolean auth2 = UserUtil.checkAuthentication(url, "GET");
                if (!auth2) {
                    auth2 = SysAttBorrowValidatorFactory.auth(method, fdId);
                }
                if (!auth2) {
                    attMain = (SysAttMain)this.sysAttMainService.findByPrimaryKey(fdId);
                    auth2 = this.checkBusinessModuleBorrowAuth(validatorContext, attMain);
                }
                return auth2;
            }
            final IAuthenticationValidateCore authenticationValidateCore = (IAuthenticationValidateCore)SpringBeanUtil.getBean("authenticationValidateCore");
            if ("true".equals(validatorContext.getParameter("v"))) {
                url = this.processModelingUrl(mainModel, url);
                return UserUtil.checkAuthentication(url, "GET");
            }
            return UserUtil.checkAuthentication(url, "GET") || authenticationValidateCore.excuteValidator(validatorContext, "lbpmCurHandlerValidator(method=edit)");
        }
    }
    
    private boolean isReadDownload(final String attType, final String contentType, final String fileName) {
        final boolean flag = contentType.contains("image") || "pic".equals(attType) || contentType.contains("text") || "text".equals(attType) || contentType.contains("pdf") || "pdf".equals(attType) || contentType.contains("ofd") || (contentType.contains("octet") && fileName.contains("ofd")) || "ofd".equals(attType) || contentType.contains("html") || "html".equals(attType);
        return flag;
    }
    
    private String processModelingUrl(final IBaseModel mainModel, final String downLoadUrl) throws Exception {
        final String modelClassName = ModelUtil.getModelClassName((Object)mainModel);
        String modelingUrl = downLoadUrl;
        if ("com.landray.kmss.sys.modeling.main.model.ModelingAppModelMain".equals(modelClassName) || "com.landray.kmss.sys.modeling.main.model.ModelingAppSimpleMain".equals(modelClassName)) {
            modelingUrl = StringUtil.linkString(modelingUrl, "&", "fdAppModelId=" + PropertyUtils.getProperty((Object)mainModel, "fdModel.fdId"));
        }
        return modelingUrl;
    }
    
    private boolean checkKmsKnowledgeBorrowAuth(final String fdMethod, final String fdModelId) {
        boolean result = false;
        if (moduleExist("/kms/knowledge")) {
            try {
                final String authFactoryName = "com.landray.kmss.kms.knowledge.borrow.service.spring.KmsKnowledgeBorrowValidatorFactory";
                final Class clazz = ClassUtils.forName(authFactoryName, (ClassLoader)null);
                final Method method = ReflectionUtils.findMethod(clazz, "auth", new Class[] { String.class, String.class });
                ReflectionUtils.makeAccessible(method);
                result = (boolean)method.invoke(null, fdMethod, fdModelId);
            }
            catch (final Exception e) {
                this.logger.error("知识借阅权限出现错误", e);
            }
        }
        return result;
    }
    
    private boolean checkBusinessModuleBorrowAuth(final ValidatorRequestContext validatorContext, final SysAttMain sysAttMain) throws Exception {
        final IExtension extension = Plugin.getExtension("com.landray.kmss.sys.attachment.borrow.business.module.validator", sysAttMain.getFdModelName(), "validator");
        if (extension != null) {
            final Object validator = Plugin.getParamValue(extension, "bean");
            if (validator instanceof ISysAttBorrowBusinessModuleValidator) {
                try {
                    final ISysAttBorrowBusinessModuleValidator businessModuleValidator = (ISysAttBorrowBusinessModuleValidator)validator;
                    return businessModuleValidator.validate(validatorContext, sysAttMain);
                }
                catch (final Exception e) {
                    this.logger.error("业务模块借阅权限出现错误", e);
                }
            }
        }
        return false;
    }
    
    public static boolean moduleExist(final String path) {
        final Boolean exist = new File(String.valueOf(PluginConfigLocationsUtil.getKmssConfigPath()) + path).exists();
        return exist;
    }
    
    private boolean checkModelAuth(final IBaseModel baseModel, final String signalField, final String listField, final String attType) throws Exception {
        if (baseModel == null) {
            return false;
        }
        if ("pic".equals(attType)) {
            return true;
        }
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("开始校验附件的主model的开关,signalField=" + signalField);
        }
        Object value = null;
        try {
            value = PropertyUtils.getProperty((Object)baseModel, signalField);
        }
        catch (final NoSuchMethodException ex) {}
        if (value != null && (boolean)value) {
            return false;
        }
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("开始校验附件的主model的list,signalField=" + signalField);
        }
        Object values = null;
        try {
            values = PropertyUtils.getProperty((Object)baseModel, listField);
        }
        catch (final NoSuchMethodException ex2) {}
        if (values != null) {
            final List list = (List)values;
            if (!list.isEmpty()) {
                return UserUtil.checkUserModels(list);
            }
            if (baseModel instanceof ExtendAuthModel) {
                final ExtendAuthModel extendAuthModel = (ExtendAuthModel)baseModel;
                final SysOrgPerson creator = extendAuthModel.getDocCreator();
                if (BooleanUtils.isTrue(creator.getFdIsExternal())) {
                    final SysOrgElement parent = creator.getFdParent();
                    if (parent != null) {
                        return UserUtil.checkUserModels(Arrays.asList((Object[])new SysOrgElement[] { parent }));
                    }
                }
            }
            else if (baseModel instanceof ExtendAuthTmpModel) {
                final ExtendAuthTmpModel extendAuthTmpModel = (ExtendAuthTmpModel)baseModel;
                final SysOrgPerson creator = extendAuthTmpModel.getDocCreator();
                if (BooleanUtils.isTrue(creator.getFdIsExternal())) {
                    final SysOrgElement parent = creator.getFdParent();
                    if (parent != null) {
                        return UserUtil.checkUserModels(Arrays.asList((Object[])new SysOrgElement[] { parent }));
                    }
                }
            }
        }
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("为限制权限,获取附件的主model的URL权限");
        }
        if (StringUtil.isNull(ModelUtil.getModelUrl((Object)baseModel))) {
            this.logger.warn(String.valueOf(ModelUtil.getModelClassName((Object)baseModel)) + "对应的数据字典没有配置URL，附件权限校验失败！");
            return false;
        }
        String url = ModelUtil.getModelUrl((Object)baseModel);
        if ("authAttNodownload".equals(signalField) && baseModel instanceof ExtendAuthModel) {
            final List downs = ((ExtendAuthModel)baseModel).getAuthAttDownloads();
            if (downs.size() > 0) {
                url = StringUtil.replace(url, "method=view", "method=edit");
            }
        }
        if ("authAttNocopy".equals(signalField) && baseModel instanceof ExtendAuthModel) {
            final List copys = ((ExtendAuthModel)baseModel).getAuthAttCopys();
            if (copys.size() > 0) {
                url = StringUtil.replace(url, "method=view", "method=edit");
            }
        }
        if (baseModel instanceof ExtendAuthModel) {
            final ExtendAuthModel model = (ExtendAuthModel)baseModel;
            return model.getDocCreator().getFdIsExternal().equals(UserUtil.getUser().getFdIsExternal()) && UserUtil.checkAuthentication(url, "GET");
        }
        if (baseModel instanceof ExtendAuthTmpModel) {
            final ExtendAuthTmpModel model2 = (ExtendAuthTmpModel)baseModel;
            return model2.getDocCreator().getFdIsExternal().equals(UserUtil.getUser().getFdIsExternal()) && UserUtil.checkAuthentication(url, "GET");
        }
        return UserUtil.checkAuthentication(url, "GET");
    }
    
    public void setSysAttMainService(final ISysAttMainCoreInnerService sysAttMainService) {
        this.sysAttMainService = sysAttMainService;
    }
}
